package org.implab.gradle.containers.tasks;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.gradle.api.DefaultTask;
import org.gradle.api.logging.Logger;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.TaskAction;
import org.implab.gradle.Utils;
import org.implab.gradle.containers.ContainerExtension;

public abstract class CliTask extends DefaultTask {

    private final Property<String> cliCmd;

    private final Property<File> workingDir;

    public CliTask() {
        cliCmd = getProject().getObjects().property(String.class);
        cliCmd.convention(getContainerExtension().getCliCmd());

        workingDir = getProject().getObjects().property(File.class);
    }

    @Input
    public Property<String> getCliCmd() {
        return cliCmd;
    }

    public void setCliCmd(String cliCmd) {
        this.cliCmd.set(cliCmd);
    }

    @Internal
    protected ContainerExtension getContainerExtension() {
        return getProject().getExtensions().getByType(ContainerExtension.class);
    }

    @TaskAction
    public void Run() throws Exception {
        final Logger log = getLogger();

        List<String> args = new ArrayList<>();
        preparingCommand(args);

        ProcessBuilder builder = new ProcessBuilder(args);
        if (workingDir.isPresent())
            builder.directory(workingDir.get());

        log.info("Starting: {}", builder.command());

        Process p = builder.start();

        processStarted(p);

        if (log.isErrorEnabled()) {
            Utils.redirectIO(p.getErrorStream(), log::error);
        }

        int code = p.waitFor();
        if (code != 0)
            throw new Exception("Process exited with the error: " + code);
    }

    protected void preparingCommand(List<String> commandLine) {
        commandLine.add(cliCmd.get());
    }

    protected void setWorkingDir(Provider<File> workingDir) {
        this.workingDir.set(workingDir);
    }

    protected void setWorkingDir(File workingDir) {
        this.workingDir.set(workingDir);
    }


    /** Invoked after the process is started, can be used to handle the process output.
     * 
     * Default implemetation redirects output to the logger as INFO messages.
     * 
     * @param p The current process.
     */
    protected void processStarted(Process p) {
        final Logger log = getLogger();
        Utils.redirectIO(p.getInputStream(), log::info);
    }
}
